import 'dart:async';
import 'package:bloc/bloc.dart';
import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/roles_and_permission/model/roles_user_model.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/users_table/bloc/user_table_event.dart';
import 'package:syncrow_web/pages/roles_and_permission/users_page/users_table/bloc/user_table_state.dart';
import 'package:syncrow_web/services/user_permission.dart';

class UserTableBloc extends Bloc<UserTableEvent, UserTableState> {
  UserTableBloc() : super(TableInitial()) {
    on<GetUsers>(_getUsers);
    on<ChangeUserStatus>(_changeUserStatus);
    on<SortUsersByNameAsc>(_toggleSortUsersByNameAsc);
    on<SortUsersByNameDesc>(_toggleSortUsersByNameDesc);
    on<DateOldestToNewestEvent>(_toggleSortUsersByDateOldestToNewest);
    on<DateNewestToOldestEvent>(_toggleSortUsersByDateNewestToOldest);
    on<SearchUsers>(_searchUsers);
    on<ChangePage>(_handlePageChange);
    on<FilterUsersByRoleEvent>(_filterUsersByRole);
    on<FilterUsersByJobEvent>(_filterUsersByJobTitle);
    on<FilterUsersByCreatedEvent>(_filterUsersByCreated);
    on<FilterUsersByDeActevateEvent>(_filterUserActevate);
    on<DeleteUserEvent>(_deleteUser);
  }
  int itemsPerPage = 10;
  int currentPage = 1;
  List<RolesUserModel> users = [];
  List<RolesUserModel> initialUsers = [];
  String currentSortOrder = '';
  String currentSortOrderDate = '';
  List<String> roleTypes = [];
  List<String> jobTitle = [];
  List<String> createdBy = [];
  List<String> deActivate = [];

  Future<void> _getUsers(GetUsers event, Emitter<UserTableState> emit) async {
    emit(UsersLoadingState());
    try {
      roleTypes.clear();
      jobTitle.clear();
      createdBy.clear();
      deActivate.clear();
      users = await UserPermissionApi().fetchUsers();
      for (var user in users) {
        roleTypes.add(user.roleType.toString());
      }
      for (var user in users) {
        jobTitle.add(user.jobTitle.toString());
      }
      for (var user in users) {
        createdBy.add(user.invitedBy.toString());
      }
      for (var user in users) {
        deActivate.add(user.status.toString());
      }
      roleTypes = roleTypes.toSet().toList();
      jobTitle = jobTitle.toSet().toList();
      createdBy = createdBy.toSet().toList();
      deActivate = deActivate.toSet().toList();

      users.sort((a, b) {
        final dateA = _parseDateTime(a.createdDate);
        final dateB = _parseDateTime(b.createdDate);
        return dateB.compareTo(dateA);
      });
      initialUsers = List.from(users);
      _handlePageChange(ChangePage(1), emit);
      emit(UsersLoadedState(users: users));
    } catch (e) {
      emit(ErrorState(e.toString()));
    }
  }

  Future<void> _deleteUser(
      DeleteUserEvent event, Emitter<UserTableState> emit) async {
    emit(UsersLoadingState());
    try {
      bool res = await UserPermissionApi().deleteUserById(event.userId);
      if (res == true) {
        Navigator.of(event.context).pop(true);
      } else {
        emit(const ErrorState('Something error'));
      }
      emit(UsersLoadedState(users: users));
    } catch (e) {
      emit(ErrorState(e.toString()));
    }
  }

  Future<void> _changeUserStatus(
      ChangeUserStatus event, Emitter<UserTableState> emit) async {
    try {
      emit(UsersLoadingState());
      bool res = await UserPermissionApi().changeUserStatusById(
          event.userId, event.newStatus == "disabled" ? true : false);
      if (res == true) {
        add(const GetUsers());
        // users = users.map((user) {
        //   if (user.uuid == event.userId) {
        //     return RolesUserModel(
        //       uuid: user.uuid,
        //       createdAt: user.createdAt,
        //       email: user.email,
        //       firstName: user.firstName,
        //       lastName: user.lastName,
        //       roleType: user.roleType,
        //       status: event.newStatus,
        //       isEnabled: event.newStatus == "disabled" ? false : true,
        //       invitedBy: user.invitedBy,
        //       phoneNumber: user.phoneNumber,
        //       jobTitle: user.jobTitle,
        //       createdDate: user.createdDate,
        //       createdTime: user.createdTime,
        //     );
        //   }
        //   return user;
        // }).toList();
      }
      emit(UsersLoadedState(users: users));
    } catch (e) {
      emit(ErrorState(e.toString()));
    }
  }

  void _toggleSortUsersByNameAsc(
      SortUsersByNameAsc event, Emitter<UserTableState> emit) {
    if (currentSortOrder == "Asc") {
      emit(UsersLoadingState());
      currentSortOrder = "";
      users = List.from(initialUsers);
      emit(UsersLoadedState(users: users));
    } else {
      emit(UsersLoadingState());
      currentSortOrder = "Asc";
      users.sort((a, b) => a.firstName!.compareTo(b.firstName!));
      emit(UsersLoadedState(users: users));
    }
  }

  void _toggleSortUsersByNameDesc(
      SortUsersByNameDesc event, Emitter<UserTableState> emit) {
    if (currentSortOrder == "Desc") {
      emit(UsersLoadingState());
      currentSortOrder = "";
      users = List.from(initialUsers); // Reset to saved initial state
      emit(UsersLoadedState(users: users));
    } else {
      // Sort descending
      emit(UsersLoadingState());
      currentSortOrder = "Desc";
      users.sort((a, b) => b.firstName!.compareTo(a.firstName!));
      emit(UsersLoadedState(users: users));
    }
  }

  void _toggleSortUsersByDateNewestToOldest(
      DateNewestToOldestEvent event, Emitter<UserTableState> emit) {
    if (currentSortOrderDate == "NewestToOldest") {
      emit(UsersLoadingState());
      currentSortOrder = "";
      currentSortOrderDate = "";
      users = List.from(initialUsers);
      emit(UsersLoadedState(users: users));
    } else {
      emit(UsersLoadingState());
      users.sort((a, b) {
        final dateA = _parseDateTime(a.createdDate);
        final dateB = _parseDateTime(b.createdDate);
        return dateB.compareTo(dateA);
      });
      emit(UsersLoadedState(users: users));
    }
  }

  void _toggleSortUsersByDateOldestToNewest(
      DateOldestToNewestEvent event, Emitter<UserTableState> emit) {
    if (currentSortOrderDate == "OldestToNewest") {
      emit(UsersLoadingState());
      currentSortOrder = "";
      currentSortOrderDate = "";
      users = List.from(initialUsers);
      emit(UsersLoadedState(users: users));
    } else {
      emit(UsersLoadingState());
      users.sort((a, b) {
        final dateA = _parseDateTime(a.createdDate);
        final dateB = _parseDateTime(b.createdDate);
        return dateA.compareTo(dateB);
      });
      emit(UsersLoadedState(users: users));
    }
  }

  DateTime _parseDateTime(String date) {
    try {
      final dateParts = date.split('/');
      final day = int.parse(dateParts[0]);
      final month = int.parse(dateParts[1]);
      final year = int.parse(dateParts[2]);
      return DateTime(year, month, day);
    } catch (e) {
      throw FormatException('Invalid date or time format: $date ');
    }
  }

  Future<void> _searchUsers(
      SearchUsers event, Emitter<UserTableState> emit) async {
    try {
      final query = event.query.toLowerCase();
      final filteredUsers = initialUsers.where((user) {
        final fullName = "${user.firstName} ${user.lastName}".toLowerCase();
        final email = user.email?.toLowerCase() ?? "";
        return fullName.contains(query) || email.contains(query);
      }).toList();
      emit(UsersLoadedState(users: filteredUsers));
    } catch (e) {
      emit(ErrorState(e.toString()));
    }
  }

  void _paginateUsers(
      int pageNumber, int itemsPerPage, Emitter<UserTableState> emit) {
    final startIndex = (pageNumber - 1) * itemsPerPage;
    final endIndex = startIndex + itemsPerPage;
    if (startIndex >= users.length) {
      emit(UsersLoadedState(users: const []));
      return;
    }
    final paginatedUsers = users.sublist(
      startIndex,
      endIndex > users.length ? users.length : endIndex,
    );
    emit(UsersLoadedState(users: paginatedUsers));
  }

  void _handlePageChange(ChangePage event, Emitter<UserTableState> emit) {
    final itemsPerPage = 10;
    final startIndex = (event.pageNumber - 1) * itemsPerPage;
    final endIndex = startIndex + itemsPerPage;
    if (startIndex >= users.length) {
      emit(UsersLoadedState(users: []));
      return;
    }
    final paginatedUsers = users.sublist(
      startIndex,
      endIndex > users.length ? users.length : endIndex,
    );
    emit(UsersLoadedState(users: paginatedUsers));
  }

  void _filterUsersByRole(
      FilterUsersByRoleEvent event, Emitter<UserTableState> emit) {
    emit(UsersLoadingState());
    final filteredUsers = initialUsers.where((user) {
      return event.selectedRoles.contains(user.roleType);
    }).toList();
    emit(UsersLoadedState(users: filteredUsers));
  }

  void _filterUsersByJobTitle(
      FilterUsersByJobEvent event, Emitter<UserTableState> emit) {
    emit(UsersLoadingState());
    final filteredUsers = users.where((user) {
      return event.selectedJob.contains(user.jobTitle);
    }).toList();
    emit(UsersLoadedState(users: filteredUsers));
  }

  void _filterUsersByCreated(
      FilterUsersByCreatedEvent event, Emitter<UserTableState> emit) {
    emit(UsersLoadingState());
    final filteredUsers = initialUsers.where((user) {
      return event.selectedCreatedBy.contains(user.invitedBy);
    }).toList();
    emit(UsersLoadedState(users: filteredUsers));
  }

  void _filterUserActevate(
      FilterUsersByDeActevateEvent event, Emitter<UserTableState> emit) {
    emit(UsersLoadingState());
    final filteredUsers = initialUsers.where((user) {
      return event.selectedActivate.contains(user.status);
    }).toList();
    emit(UsersLoadedState(users: filteredUsers));
  }
}
